@@ -34,36 +34,34 @@ import Distribution.Client.Setup
          , ConfigExFlags(..), configureExOptions, defaultConfigExFlags
          , InstallFlags(..), installOptions, defaultInstallFlags
          , UploadFlags(..), uploadCommand
+         , ReportFlags(..), reportCommand
          , showRepo, parseRepo )
 
 import Distribution.Simple.Setup
          ( ConfigFlags(..), configureOptions, defaultConfigFlags
-         , Flag, toFlag, flagToMaybe, fromFlagOrDefault, flagToList )
+         , installDirsOptions
+         , Flag, toFlag, flagToMaybe, fromFlagOrDefault )
 import Distribution.Simple.InstallDirs
          ( InstallDirs(..), defaultInstallDirs
-         , PathTemplate, toPathTemplate, fromPathTemplate )
+         , PathTemplate, toPathTemplate )
 import Distribution.ParseUtils
          ( FieldDescr(..), liftField
          , ParseResult(..), locatedErrorMsg, showPWarning
          , readFields, warning, lineNo
-         , simpleField, listField, parseFilePathQ, showFilePath, parseTokenQ )
+         , simpleField, listField, parseFilePathQ, parseTokenQ )
 import qualified Distribution.ParseUtils as ParseUtils
          ( Field(..) )
 import qualified Distribution.Text as Text
          ( Text(..) )
-import Distribution.ReadE
-         ( readP_to_E )
 import Distribution.Simple.Command
          ( CommandUI(commandOptions), commandDefaultFlags, ShowOrParseArgs(..)
-         , viewAsFieldDescr, OptionField, option, reqArg )
+         , viewAsFieldDescr )
 import Distribution.Simple.Program
          ( defaultProgramConfiguration )
 import Distribution.Simple.Utils
          ( notice, warn, lowercase )
 import Distribution.Compiler
          ( CompilerFlavor(..), defaultCompilerFlavor )
-import Distribution.System
-         ( OS(Windows), buildOS )
 import Distribution.Verbosity
          ( Verbosity, normal )
 
@@ -104,7 +102,8 @@ data SavedConfig = SavedConfig {
     savedConfigureExFlags  :: ConfigExFlags,
     savedUserInstallDirs   :: InstallDirs (Flag PathTemplate),
     savedGlobalInstallDirs :: InstallDirs (Flag PathTemplate),
-    savedUploadFlags       :: UploadFlags
+    savedUploadFlags       :: UploadFlags,
+    savedReportFlags       :: ReportFlags
   }
 
 instance Monoid SavedConfig where
@@ -115,7 +114,8 @@ instance Monoid SavedConfig where
     savedConfigureExFlags  = mempty,
     savedUserInstallDirs   = mempty,
     savedGlobalInstallDirs = mempty,
-    savedUploadFlags       = mempty
+    savedUploadFlags       = mempty,
+    savedReportFlags       = mempty
   }
   mappend a b = SavedConfig {
     savedGlobalFlags       = combine savedGlobalFlags,
@@ -124,7 +124,8 @@ instance Monoid SavedConfig where
     savedConfigureExFlags  = combine savedConfigureExFlags,
     savedUserInstallDirs   = combine savedUserInstallDirs,
     savedGlobalInstallDirs = combine savedGlobalInstallDirs,
-    savedUploadFlags       = combine savedUploadFlags
+    savedUploadFlags       = combine savedUploadFlags,
+    savedReportFlags       = combine savedReportFlags
   }
     where combine field = field a `mappend` field b
 
@@ -158,6 +159,8 @@ updateInstallDirs userInstallFlag
 baseSavedConfig :: IO SavedConfig
 baseSavedConfig = do
   userPrefix <- defaultCabalDir
+  logsDir    <- defaultLogsDir
+  worldFile  <- defaultWorldFile
   return mempty {
     savedConfigureFlags  = mempty {
       configHcFlavor     = toFlag defaultCompiler,
@@ -166,6 +169,10 @@ baseSavedConfig = do
     },
     savedUserInstallDirs = mempty {
       prefix             = toFlag (toPathTemplate userPrefix)
+    },
+    savedGlobalFlags = mempty {
+      globalLogsDir      = toFlag logsDir,
+      globalWorldFile    = toFlag worldFile
     }
   }
 
@@ -179,10 +186,12 @@ initialSavedConfig :: IO SavedConfig
 initialSavedConfig = do
   cacheDir   <- defaultCacheDir
   logsDir    <- defaultLogsDir
+  worldFile  <- defaultWorldFile
   return mempty {
     savedGlobalFlags     = mempty {
       globalCacheDir     = toFlag cacheDir,
-      globalRemoteRepos  = [defaultRemoteRepo]
+      globalRemoteRepos  = [defaultRemoteRepo],
+      globalWorldFile    = toFlag worldFile
     },
     savedInstallFlags    = mempty {
       installSummaryFile = [toPathTemplate (logsDir </> "build.log")],
@@ -190,6 +199,8 @@ initialSavedConfig = do
     }
   }
 
+--TODO: misleading, there's no way to override this default
+--      either make it possible or rename to simply getCabalDir.
 defaultCabalDir :: IO FilePath
 defaultCabalDir = getAppUserDataDirectory "cabal"
 
@@ -208,15 +219,19 @@ defaultLogsDir = do
   dir <- defaultCabalDir
   return $ dir </> "logs"
 
+-- | Default position of the world file
+defaultWorldFile :: IO FilePath
+defaultWorldFile = do
+  dir <- defaultCabalDir
+  return $ dir </> "world"
+
 defaultCompiler :: CompilerFlavor
 defaultCompiler = fromMaybe GHC defaultCompilerFlavor
 
 defaultUserInstall :: Bool
-defaultUserInstall = case buildOS of
-  -- We do global installs by default on Windows
-  Windows -> False
-  -- and per-user installs by default everywhere else
-  _       -> True
+defaultUserInstall = True
+-- We do per-user installs by default on all platforms. We used to default to
+-- global installs on Windows but that no longer works on Windows Vista or 7.
 
 defaultRemoteRepo :: RemoteRepo
 defaultRemoteRepo = RemoteRepo name uri
@@ -313,7 +328,8 @@ commentSavedConfig = do
     },
     savedUserInstallDirs   = fmap toFlag userInstallDirs,
     savedGlobalInstallDirs = fmap toFlag globalInstallDirs,
-    savedUploadFlags       = commandDefaultFlags uploadCommand
+    savedUploadFlags       = commandDefaultFlags uploadCommand,
+    savedReportFlags       = commandDefaultFlags reportCommand
   }
 
 -- | All config file fields.
@@ -349,6 +365,10 @@ configFieldDescriptions =
        (commandOptions uploadCommand ParseArgs)
        ["verbose", "check"] []
 
+  ++ toSavedConfig liftReportFlag
+       (commandOptions reportCommand ParseArgs)
+       ["verbose"] []
+
   where
     toSavedConfig lift options exclusions replacements =
       [ lift (fromMaybe field replacement)
@@ -419,6 +439,10 @@ liftUploadFlag :: FieldDescr UploadFlags -> FieldDescr SavedConfig
 liftUploadFlag = liftField
   savedUploadFlags (\flags conf -> conf { savedUploadFlags = flags })
 
+liftReportFlag :: FieldDescr ReportFlags -> FieldDescr SavedConfig
+liftReportFlag = liftField
+  savedReportFlags (\flags conf -> conf { savedReportFlags = flags })
+
 parseConfig :: SavedConfig -> String -> ParseResult SavedConfig
 parseConfig initial = \str -> do
   fields <- readFields str
@@ -506,64 +530,3 @@ ppSection name arg fields def cur =
 installDirsFields :: [FieldDescr (InstallDirs (Flag PathTemplate))]
 installDirsFields = map viewAsFieldDescr installDirsOptions
 
---TODO: this is now exported in Cabal-1.5
-installDirsOptions :: [OptionField (InstallDirs (Flag PathTemplate))]
-installDirsOptions =
-  [ option "" ["prefix"]
-      "bake this prefix in preparation of installation"
-      prefix (\v flags -> flags { prefix = v })
-      installDirArg
-
-  , option "" ["bindir"]
-      "installation directory for executables"
-      bindir (\v flags -> flags { bindir = v })
-      installDirArg
-
-  , option "" ["libdir"]
-      "installation directory for libraries"
-      libdir (\v flags -> flags { libdir = v })
-      installDirArg
-
-  , option "" ["libsubdir"]
-      "subdirectory of libdir in which libs are installed"
-      libsubdir (\v flags -> flags { libsubdir = v })
-      installDirArg
-
-  , option "" ["libexecdir"]
-      "installation directory for program executables"
-      libexecdir (\v flags -> flags { libexecdir = v })
-      installDirArg
-
-  , option "" ["datadir"]
-      "installation directory for read-only data"
-      datadir (\v flags -> flags { datadir = v })
-      installDirArg
-
-  , option "" ["datasubdir"]
-      "subdirectory of datadir in which data files are installed"
-      datasubdir (\v flags -> flags { datasubdir = v })
-      installDirArg
-
-  , option "" ["docdir"]
-      "installation directory for documentation"
-      docdir (\v flags -> flags { docdir = v })
-      installDirArg
-
-  , option "" ["htmldir"]
-      "installation directory for HTML documentation"
-      htmldir (\v flags -> flags { htmldir = v })
-      installDirArg
-
-  , option "" ["haddockdir"]
-      "installation directory for haddock interfaces"
-      haddockdir (\v flags -> flags { haddockdir = v })
-      installDirArg
-  ]
-  where
-    installDirArg _sf _lf d get set =
-      reqArgFlag "DIR" _sf _lf d
-        (fmap fromPathTemplate . get) (set . fmap toPathTemplate)
-
-    reqArgFlag ad = reqArg ad (fmap toFlag (readP_to_E err parseFilePathQ))
-                              (map (show . showFilePath) . flagToList)
-      where err _ = "paths with spaces must use Haskell String syntax"
