@@ -14,16 +14,7 @@ module Distribution.Client.Configure (
     configure,
   ) where
 
-import Data.Monoid
-         ( Monoid(mempty) )
-import qualified Data.Map as Map
-
 import Distribution.Client.Dependency
-         ( resolveDependenciesWithProgress
-         , PackageConstraint(..)
-         , PackagesPreference(..), PackagesPreferenceDefault(..)
-         , PackagePreference(..)
-         , Progress(..), foldProgress, )
 import qualified Distribution.Client.InstallPlan as InstallPlan
 import Distribution.Client.InstallPlan (InstallPlan)
 import Distribution.Client.IndexUtils as IndexUtils
@@ -31,8 +22,6 @@ import Distribution.Client.IndexUtils as IndexUtils
 import Distribution.Client.Setup
          ( ConfigExFlags(..), configureCommand, filterConfigureFlags )
 import Distribution.Client.Types as Available
-         ( AvailablePackage(..), AvailablePackageSource(..), Repo(..)
-         , AvailablePackageDb(..), ConfiguredPackage(..), InstalledPackage )
 import Distribution.Client.SetupWrapper
          ( setupWrapper, SetupScriptOptions(..), defaultSetupScriptOptions )
 
@@ -42,26 +31,26 @@ import Distribution.Simple.Compiler
 import Distribution.Simple.Program (ProgramConfiguration )
 import Distribution.Simple.Setup
          ( ConfigFlags(..), toFlag, flagToMaybe, fromFlagOrDefault )
-import qualified Distribution.Client.PackageIndex as PackageIndex
 import Distribution.Client.PackageIndex (PackageIndex)
 import Distribution.Simple.Utils
          ( defaultPackageDesc )
 import Distribution.Package
-         ( PackageName, packageName, packageVersion
-         , Package(..), Dependency(..), thisPackageVersion )
+         ( Package(..), packageName, Dependency(..), thisPackageVersion )
 import Distribution.PackageDescription.Parse
          ( readPackageDescription )
 import Distribution.PackageDescription.Configuration
          ( finalizePackageDescription )
 import Distribution.Version
-         ( VersionRange, anyVersion, thisVersion )
+         ( anyVersion, thisVersion )
 import Distribution.Simple.Utils as Utils
-         ( notice, info, die )
+         ( notice, info, debug, die )
 import Distribution.System
          ( Platform, buildPlatform )
 import Distribution.Verbosity as Verbosity
          ( Verbosity )
 
+import Data.Monoid (Monoid(..))
+
 -- | Configure the package found in the local directory
 configure :: Verbosity
           -> PackageDBStack
@@ -82,8 +71,8 @@ configure verbosity packageDBs repos comp conf
                                installed available
 
   notice verbosity "Resolving dependencies..."
-  maybePlan <- foldProgress (\message rest -> info verbosity message >> rest)
-                            (return . Left) (return . Right) progress
+  maybePlan <- foldProgress logMsg (return . Left) (return . Right)
+                            progress
   case maybePlan of
     Left message -> do
       info verbosity message
@@ -91,7 +80,7 @@ configure verbosity packageDBs repos comp conf
         configureCommand (const configFlags) extraArgs
 
     Right installPlan -> case InstallPlan.ready installPlan of
-      [pkg@(ConfiguredPackage (AvailablePackage _ _ LocalUnpackedPackage) _ _)] ->
+      [pkg@(ConfiguredPackage (AvailablePackage _ _ (LocalUnpackedPackage _)) _ _)] ->
         configurePackage verbosity
           (InstallPlan.planPlatform installPlan)
           (InstallPlan.planCompiler installPlan)
@@ -113,7 +102,7 @@ configure verbosity packageDBs repos comp conf
                            then packageDBs
                            else packageDBs ++ [UserPackageDB],
       usePackageIndex  = if UserPackageDB `elem` packageDBs
-                           then index
+                           then Just index
                            else Nothing,
       useProgramConfig = conf,
       useDistPref      = fromFlagOrDefault
@@ -123,54 +112,51 @@ configure verbosity packageDBs repos comp conf
       useWorkingDir    = Nothing
     }
 
+    logMsg message rest = debug verbosity message >> rest
+
 -- | Make an 'InstallPlan' for the unpacked package in the current directory,
 -- and all its dependencies.
 --
 planLocalPackage :: Verbosity -> Compiler
                  -> ConfigFlags -> ConfigExFlags
-                 -> Maybe (PackageIndex InstalledPackage)
+                 -> PackageIndex InstalledPackage
                  -> AvailablePackageDb
                  -> IO (Progress String String InstallPlan)
 planLocalPackage verbosity comp configFlags configExFlags installed
   (AvailablePackageDb _ availablePrefs) = do
   pkg <- readPackageDescription verbosity =<< defaultPackageDesc verbosity
-  let -- The trick is, we add the local package to the available index and
-      -- remove it from the installed index. Then we ask to resolve a
-      -- dependency on exactly that package. So the resolver ends up having
-      -- to pick the local package.
-      available' = PackageIndex.insert localPkg mempty
-      installed' = PackageIndex.deletePackageId (packageId localPkg) `fmap` installed
+
+  let -- We create a local package and ask to resolve a dependency on it
       localPkg = AvailablePackage {
         packageInfoId                = packageId pkg,
         Available.packageDescription = pkg,
-        packageSource                = LocalUnpackedPackage
+        packageSource                = LocalUnpackedPackage "."
       }
-      targets     = [packageName pkg]
-      constraints = [PackageVersionConstraint (packageName pkg)
-                       (thisVersion (packageVersion pkg))
-                    ,PackageFlagsConstraint   (packageName pkg)
-                       (configConfigurationsFlags configFlags)]
-                 ++ [ PackageVersionConstraint name ver
-                    | Dependency name ver <- configConstraints configFlags ]
-      preferences = mergePackagePrefs PreferLatestForSelected
-                                      availablePrefs configExFlags
 
-  return $ resolveDependenciesWithProgress buildPlatform (compilerId comp)
-             installed' available' preferences constraints targets
+      resolverParams =
 
+          addPreferences
+            -- preferences from the config file or command line
+            [ PackageVersionPreference name ver
+            | Dependency name ver <- configPreferences configExFlags ]
+
+        . addConstraints
+            -- version constraints from the config file or command line
+            [ PackageVersionConstraint name ver
+            | Dependency name ver <- configConstraints configFlags ]
+
+        . addConstraints
+            -- package flags from the config file or command line
+            [ PackageFlagsConstraint (packageName pkg)
+                                     (configConfigurationsFlags configFlags) ]
+
+        $ standardInstallPolicy
+            installed
+            (AvailablePackageDb mempty availablePrefs)
+            [SpecificSourcePackage localPkg]
+
+  return (resolveDependencies buildPlatform (compilerId comp) resolverParams)
 
-mergePackagePrefs :: PackagesPreferenceDefault
-                  -> Map.Map PackageName VersionRange
-                  -> ConfigExFlags
-                  -> PackagesPreference
-mergePackagePrefs defaultPref availablePrefs configExFlags =
-  PackagesPreference defaultPref $
-       -- The preferences that come from the hackage index
-       [ PackageVersionPreference name ver
-       | (name, ver) <- Map.toList availablePrefs ]
-       -- additional preferences from the config file or command line
-    ++ [ PackageVersionPreference name ver
-       | Dependency name ver <- configPreferences configExFlags ]
 
 -- | Call an installer for an 'AvailablePackage' but override the configure
 -- flags with the ones given by the 'ConfiguredPackage'. In particular the
