@@ -4,7 +4,7 @@
 -- Copyright   :  (c) Duncan Coutts 2008
 -- License     :  BSD-like
 --
--- Maintainer  :  duncan@haskell.org
+-- Maintainer  :  duncan@community.haskell.org
 -- Stability   :  provisional
 -- Portability :  portable
 --
@@ -22,6 +22,7 @@ module Distribution.Client.InstallPlan (
   ready,
   completed,
   failed,
+  remove,
 
   -- ** Query functions
   planPlatform,
@@ -54,8 +55,9 @@ import Distribution.Version
          ( Version, withinRange )
 import Distribution.PackageDescription
          ( GenericPackageDescription(genPackageFlags)
-         , PackageDescription(buildDepends)
          , Flag(flagName), FlagName(..) )
+import Distribution.Client.PackageUtils
+         ( externalBuildDepends )
 import Distribution.PackageDescription.Configuration
          ( finalizePackageDescription )
 import Distribution.Client.PackageIndex
@@ -180,6 +182,21 @@ new platform compiler index =
 toList :: InstallPlan -> [PlanPackage]
 toList = PackageIndex.allPackages . planIndex
 
+-- | Remove packages from the install plan. This will result in an
+-- error if there are remaining packages that depend on any matching
+-- package. This is primarily useful for obtaining an install plan for
+-- the dependencies of a package or set of packages without actually
+-- installing the package itself, as when doing development.
+--
+remove :: (PlanPackage -> Bool)
+       -> InstallPlan
+       -> Either [PlanProblem] InstallPlan
+remove shouldRemove plan =
+    new (planPlatform plan) (planCompiler plan) newIndex
+  where
+    newIndex = PackageIndex.fromList $
+                 filter (not . shouldRemove) (toList plan)
+
 -- | The packages that are ready to be installed. That is they are in the
 -- configured state and have all their dependencies installed already.
 -- The plan is complete if the result is @[]@.
@@ -490,5 +507,5 @@ configuredPackageProblems platform comp
          platform comp
          []
          (packageDescription pkg) of
-        Right (resolvedPkg, _) -> buildDepends resolvedPkg
+        Right (resolvedPkg, _) -> externalBuildDepends resolvedPkg
         Left  _ -> error "configuredPackageInvalidDeps internal error"
