@@ -20,10 +20,10 @@ module Distribution.Client.Setup
     , updateCommand
     , upgradeCommand
     , infoCommand, InfoFlags(..)
-    , fetchCommand
+    , fetchCommand, FetchFlags(..)
     , checkCommand
     , uploadCommand, UploadFlags(..)
-    , reportCommand
+    , reportCommand, ReportFlags(..)
     , unpackCommand, UnpackFlags(..)
     , initCommand, IT.InitFlags(..)
 
@@ -50,7 +50,7 @@ import Distribution.Simple.Setup
          ( ConfigFlags(..) )
 import Distribution.Simple.Setup
          ( Flag(..), toFlag, fromFlag, flagToList, flagToMaybe
-         , optionVerbosity, trueArg )
+         , optionVerbosity, trueArg, falseArg )
 import Distribution.Simple.InstallDirs
          ( PathTemplate, toPathTemplate, fromPathTemplate )
 import Distribution.Version
@@ -65,6 +65,8 @@ import qualified Distribution.Compat.ReadP as Parse
          ( ReadP, readP_to_S, char, munch1, pfail, (+++) )
 import Distribution.Verbosity
          ( Verbosity, normal )
+import Distribution.Simple.Utils
+         ( wrapText )
 
 import Data.Char
          ( isSpace, isAlphaNum )
@@ -90,7 +92,9 @@ data GlobalFlags = GlobalFlags {
     globalConfigFile     :: Flag FilePath,
     globalRemoteRepos    :: [RemoteRepo],     -- ^Available Hackage servers.
     globalCacheDir       :: Flag FilePath,
-    globalLocalRepos     :: [FilePath]
+    globalLocalRepos     :: [FilePath],
+    globalLogsDir        :: Flag FilePath,
+    globalWorldFile      :: Flag FilePath
   }
 
 defaultGlobalFlags :: GlobalFlags
@@ -100,7 +104,9 @@ defaultGlobalFlags  = GlobalFlags {
     globalConfigFile     = mempty,
     globalRemoteRepos    = [],
     globalCacheDir       = mempty,
-    globalLocalRepos     = mempty
+    globalLocalRepos     = mempty,
+    globalLogsDir        = mempty,
+    globalWorldFile      = mempty
   }
 
 globalCommand :: CommandUI GlobalFlags
@@ -150,6 +156,16 @@ globalCommand = CommandUI {
          "The location of a local repository"
          globalLocalRepos (\v flags -> flags { globalLocalRepos = v })
          (reqArg' "DIR" (\x -> [x]) id)
+
+      ,option [] ["logs-dir"]
+         "The location to put log files"
+         globalLogsDir (\v flags -> flags { globalLogsDir = v })
+         (reqArgFlag "DIR")
+
+      ,option [] ["world-file"]
+         "The location of the world file"
+         globalWorldFile (\v flags -> flags { globalWorldFile = v })
+         (reqArgFlag "FILE")
       ]
   }
 
@@ -160,7 +176,9 @@ instance Monoid GlobalFlags where
     globalConfigFile     = mempty,
     globalRemoteRepos    = mempty,
     globalCacheDir       = mempty,
-    globalLocalRepos     = mempty
+    globalLocalRepos     = mempty,
+    globalLogsDir        = mempty,
+    globalWorldFile      = mempty
   }
   mappend a b = GlobalFlags {
     globalVersion        = combine globalVersion,
@@ -168,7 +186,9 @@ instance Monoid GlobalFlags where
     globalConfigFile     = combine globalConfigFile,
     globalRemoteRepos    = combine globalRemoteRepos,
     globalCacheDir       = combine globalCacheDir,
-    globalLocalRepos     = combine globalLocalRepos
+    globalLocalRepos     = combine globalLocalRepos,
+    globalLogsDir        = combine globalLogsDir,
+    globalWorldFile      = combine globalWorldFile
   }
     where combine field = field a `mappend` field b
 
@@ -258,19 +278,60 @@ instance Monoid ConfigExFlags where
     where combine field = field a `mappend` field b
 
 -- ------------------------------------------------------------
--- * Other commands
+-- * Fetch command
 -- ------------------------------------------------------------
 
-fetchCommand :: CommandUI (Flag Verbosity)
+data FetchFlags = FetchFlags {
+--    fetchOutput    :: Flag FilePath,
+      fetchDeps      :: Flag Bool,
+      fetchDryRun    :: Flag Bool,
+      fetchVerbosity :: Flag Verbosity
+    }
+
+defaultFetchFlags :: FetchFlags
+defaultFetchFlags = FetchFlags {
+--  fetchOutput    = mempty,
+    fetchDeps      = toFlag True,
+    fetchDryRun    = toFlag False,
+    fetchVerbosity = toFlag normal
+   }
+
+fetchCommand :: CommandUI FetchFlags
 fetchCommand = CommandUI {
     commandName         = "fetch",
-    commandSynopsis     = "Downloads packages for later installation or study.",
+    commandSynopsis     = "Downloads packages for later installation.",
     commandDescription  = Nothing,
     commandUsage        = usagePackages "fetch",
-    commandDefaultFlags = toFlag normal,
-    commandOptions      = \_ -> [optionVerbosity id const]
+    commandDefaultFlags = defaultFetchFlags,
+    commandOptions      = \_ -> [
+         optionVerbosity fetchVerbosity (\v flags -> flags { fetchVerbosity = v })
+
+--     , option "o" ["output"]
+--         "Put the package(s) somewhere specific rather than the usual cache."
+--         fetchOutput (\v flags -> flags { fetchOutput = v })
+--         (reqArgFlag "PATH")
+
+       , option [] ["dependencies", "deps"]
+           "Resolve and fetch dependencies (default)"
+           fetchDeps (\v flags -> flags { fetchDeps = v })
+           trueArg
+
+       , option [] ["no-dependencies", "no-deps"]
+           "Ignore dependencies"
+           fetchDeps (\v flags -> flags { fetchDeps = v })
+           falseArg
+
+       , option [] ["dry-run"]
+           "Do not install anything, only print what would be installed."
+           fetchDryRun (\v flags -> flags { fetchDryRun = v })
+           trueArg
+       ]
   }
 
+-- ------------------------------------------------------------
+-- * Other commands
+-- ------------------------------------------------------------
+
 updateCommand  :: CommandUI (Flag Verbosity)
 updateCommand = CommandUI {
     commandName         = "update",
@@ -284,7 +345,7 @@ updateCommand = CommandUI {
 upgradeCommand  :: CommandUI (ConfigFlags, ConfigExFlags, InstallFlags)
 upgradeCommand = configureCommand {
     commandName         = "upgrade",
-    commandSynopsis     = "Upgrades installed packages to the latest available version",
+    commandSynopsis     = "(command disabled, use install instead)",
     commandDescription  = Nothing,
     commandUsage        = usagePackages "upgrade",
     commandDefaultFlags = (mempty, mempty, mempty),
@@ -312,16 +373,62 @@ checkCommand = CommandUI {
     commandOptions      = \_ -> []
   }
 
-reportCommand :: CommandUI (Flag Verbosity)
+-- ------------------------------------------------------------
+-- * Report flags
+-- ------------------------------------------------------------
+
+data ReportFlags = ReportFlags {
+    reportUsername  :: Flag Username,
+    reportPassword  :: Flag Password,
+    reportVerbosity :: Flag Verbosity
+  }
+
+defaultReportFlags :: ReportFlags
+defaultReportFlags = ReportFlags {
+    reportUsername  = mempty,
+    reportPassword  = mempty,
+    reportVerbosity = toFlag normal
+  }
+
+reportCommand :: CommandUI ReportFlags
 reportCommand = CommandUI {
     commandName         = "report",
     commandSynopsis     = "Upload build reports to a remote server.",
-    commandDescription  = Nothing,
-    commandUsage        = \pname -> "Usage: " ++ pname ++ " report\n",
-    commandDefaultFlags = toFlag normal,
-    commandOptions      = \_ -> [optionVerbosity id const]
+    commandDescription  = Just $ \_ ->
+         "You can store your Hackage login in the ~/.cabal/config file\n",
+    commandUsage        = \pname -> "Usage: " ++ pname ++ " report [FLAGS]\n\n"
+      ++ "Flags for upload:",
+    commandDefaultFlags = defaultReportFlags,
+    commandOptions      = \_ ->
+      [optionVerbosity reportVerbosity (\v flags -> flags { reportVerbosity = v })
+
+      ,option ['u'] ["username"]
+        "Hackage username."
+        reportUsername (\v flags -> flags { reportUsername = v })
+        (reqArg' "USERNAME" (toFlag . Username)
+                            (flagToList . fmap unUsername))
+
+      ,option ['p'] ["password"]
+        "Hackage password."
+        reportPassword (\v flags -> flags { reportPassword = v })
+        (reqArg' "PASSWORD" (toFlag . Password)
+                            (flagToList . fmap unPassword))
+      ]
   }
 
+instance Monoid ReportFlags where
+  mempty = ReportFlags {
+    reportUsername  = mempty,
+    reportPassword  = mempty,
+    reportVerbosity = mempty
+  }
+  mappend a b = ReportFlags {
+    reportUsername  = combine reportUsername,
+    reportPassword  = combine reportPassword,
+    reportVerbosity = combine reportVerbosity
+  }
+    where combine field = field a `mappend` field b
+
 -- ------------------------------------------------------------
 -- * Unpack flags
 -- ------------------------------------------------------------
@@ -454,12 +561,15 @@ data InstallFlags = InstallFlags {
     installHaddockIndex :: Flag PathTemplate,
     installDryRun       :: Flag Bool,
     installReinstall    :: Flag Bool,
+    installUpgradeDeps  :: Flag Bool,
     installOnly         :: Flag Bool,
+    installOnlyDeps     :: Flag Bool,
     installRootCmd      :: Flag String,
     installSummaryFile  :: [PathTemplate],
     installLogFile      :: Flag PathTemplate,
     installBuildReports :: Flag ReportLevel,
-    installSymlinkBinDir:: Flag FilePath
+    installSymlinkBinDir:: Flag FilePath,
+    installOneShot      :: Flag Bool
   }
 
 defaultInstallFlags :: InstallFlags
@@ -468,12 +578,15 @@ defaultInstallFlags = InstallFlags {
     installHaddockIndex = Flag docIndexFile,
     installDryRun       = Flag False,
     installReinstall    = Flag False,
+    installUpgradeDeps  = Flag False,
     installOnly         = Flag False,
+    installOnlyDeps     = Flag False,
     installRootCmd      = mempty,
     installSummaryFile  = mempty,
     installLogFile      = mempty,
     installBuildReports = Flag NoReports,
-    installSymlinkBinDir= mempty
+    installSymlinkBinDir= mempty,
+    installOneShot      = Flag False
   }
   where
     docIndexFile = toPathTemplate ("$datadir" </> "doc" </> "index.html")
@@ -531,6 +644,18 @@ installOptions showOrParseArgs =
           installReinstall (\v flags -> flags { installReinstall = v })
           trueArg
 
+      , option [] ["upgrade-dependencies"]
+          "Pick the latest version for all dependencies, rather than trying to pick an installed version."
+          installUpgradeDeps (\v flags -> flags { installUpgradeDeps = v })
+          trueArg
+
+
+      , option [] ["only-dependencies"]
+          "Install only the dependencies necessary to build the given packages"
+          installOnlyDeps (\v flags -> flags { installOnlyDeps = v })
+          trueArg
+
+
       , option [] ["root-cmd"]
           "Command used to gain root privileges, when installing with --global."
           installRootCmd (\v flags -> flags { installRootCmd = v })
@@ -560,6 +685,10 @@ installOptions showOrParseArgs =
                                       (toFlag `fmap` parse))
                           (flagToList . fmap display))
 
+      , option [] ["one-shot"]
+          "Do not record the packages in the world file."
+          installOneShot (\v flags -> flags { installOneShot = v })
+          trueArg
       ] ++ case showOrParseArgs of      -- TODO: remove when "cabal install" avoids
           ParseArgs ->
             option [] ["only"]
@@ -575,24 +704,30 @@ instance Monoid InstallFlags where
     installHaddockIndex = mempty,
     installDryRun       = mempty,
     installReinstall    = mempty,
+    installUpgradeDeps  = mempty,
     installOnly         = mempty,
+    installOnlyDeps     = mempty,
     installRootCmd      = mempty,
     installSummaryFile  = mempty,
     installLogFile      = mempty,
     installBuildReports = mempty,
-    installSymlinkBinDir= mempty
+    installSymlinkBinDir= mempty,
+    installOneShot      = mempty
   }
   mappend a b = InstallFlags {
     installDocumentation= combine installDocumentation,
     installHaddockIndex = combine installHaddockIndex,
     installDryRun       = combine installDryRun,
     installReinstall    = combine installReinstall,
+    installUpgradeDeps  = combine installUpgradeDeps,
     installOnly         = combine installOnly,
+    installOnlyDeps     = combine installOnlyDeps,
     installRootCmd      = combine installRootCmd,
     installSummaryFile  = combine installSummaryFile,
     installLogFile      = combine installLogFile,
     installBuildReports = combine installBuildReports,
-    installSymlinkBinDir= combine installSymlinkBinDir
+    installSymlinkBinDir= combine installSymlinkBinDir,
+    installOneShot      = combine installOneShot
   }
     where combine field = field a `mappend` field b
 
@@ -676,25 +811,21 @@ initCommand :: CommandUI IT.InitFlags
 initCommand = CommandUI {
     commandName = "init",
     commandSynopsis = "Interactively create a .cabal file.",
-    commandUsage = \pname -> unlines
-       [ "Usage: " ++ pname ++ " init [FLAGS]"
-       , ""
-       , "Cabalise a project by creating a .cabal, Setup.lhs, and"
-       , "optionally a LICENSE file."
-       , ""
-       , "Calling init with no arguments (recommended) uses an"
-       , "interactive mode, which will try to guess as much as"
-       , "possible and prompt you for the rest.  Command-line"
-       , "arguments are provided for scripting purposes."
-       , "If you don't want interactive mode, be sure to pass"
-       , "the -n flag."
-       , ""
-       , "Flags for init:"
-       ],
-    commandDescription = Nothing,
+    commandDescription = Just $ \_ -> wrapText $
+         "Cabalise a project by creating a .cabal, Setup.hs, and "
+      ++ "optionally a LICENSE file.\n\n"
+      ++ "Calling init with no arguments (recommended) uses an "
+      ++ "interactive mode, which will try to guess as much as "
+      ++ "possible and prompt you for the rest.  Command-line "
+      ++ "arguments are provided for scripting purposes. "
+      ++ "If you don't want interactive mode, be sure to pass "
+      ++ "the -n flag.\n",
+    commandUsage = \pname ->
+         "Usage: " ++ pname ++ " init [FLAGS]\n\n"
+      ++ "Flags for init:",
     commandDefaultFlags = defaultInitFlags,
     commandOptions = \_ ->
-      [ option ['n'] ["nonInteractive"]
+      [ option ['n'] ["non-interactive"]
         "Non-interactive mode."
         IT.nonInteractive (\v flags -> flags { IT.nonInteractive = v })
         trueArg
@@ -704,22 +835,22 @@ initCommand = CommandUI {
         IT.quiet (\v flags -> flags { IT.quiet = v })
         trueArg
 
-      , option [] ["noComments"]
+      , option [] ["no-comments"]
         "Do not generate explanatory comments in the .cabal file."
         IT.noComments (\v flags -> flags { IT.noComments = v })
         trueArg
 
       , option ['m'] ["minimal"]
-        "Generate a minimal .cabal file, that is, do not include extra empty fields.  Also implies --noComments."
+        "Generate a minimal .cabal file, that is, do not include extra empty fields.  Also implies --no-comments."
         IT.minimal (\v flags -> flags { IT.minimal = v })
         trueArg
 
-      , option [] ["packageDir"]
+      , option [] ["package-dir"]
         "Root directory of the package (default = current directory)."
         IT.packageDir (\v flags -> flags { IT.packageDir = v })
         (reqArgFlag "DIRECTORY")
 
-      , option ['p'] ["packageName"]
+      , option ['p'] ["package-name"]
         "Name of the Cabal package to create."
         IT.packageName (\v flags -> flags { IT.packageName = v })
         (reqArgFlag "PACKAGE")
@@ -731,7 +862,7 @@ initCommand = CommandUI {
                                       (toFlag `fmap` parse))
                           (flagToList . fmap display))
 
-      , option [] ["cabalVersion"]
+      , option [] ["cabal-version"]
         "Required version of the Cabal library."
         IT.cabalVersion (\v flags -> flags { IT.cabalVersion = v })
         (reqArg "VERSION_RANGE" (readP_to_E ("Cannot parse Cabal version range: "++)
@@ -755,12 +886,6 @@ initCommand = CommandUI {
         IT.email (\v flags -> flags { IT.email = v })
         (reqArgFlag "EMAIL")
 
-      , option [] ["stability"]
-        "Package stability."
-        IT.stability (\v flags -> flags { IT.stability = v })
-        (reqArg' "STABILITY" (\s -> toFlag $ maybe (Left s) Right (readMaybe s))
-                             (flagToList . fmap (either id show)))
-
       , option ['u'] ["homepage"]
         "Project homepage and/or repository."
         IT.homepage (\v flags -> flags { IT.homepage = v })
@@ -777,18 +902,18 @@ initCommand = CommandUI {
         (reqArg' "CATEGORY" (\s -> toFlag $ maybe (Left s) Right (readMaybe s))
                             (flagToList . fmap (either id show)))
 
-      , option [] ["isLibrary"]
+      , option [] ["is-library"]
         "Build a library."
         IT.packageType (\v flags -> flags { IT.packageType = v })
         (noArg (Flag IT.Library))
 
-      , option [] ["isExecutable"]
+      , option [] ["is-executable"]
         "Build an executable."
         IT.packageType
         (\v flags -> flags { IT.packageType = v })
         (noArg (Flag IT.Executable))
 
-      , option ['o'] ["exposeModule"]
+      , option ['o'] ["expose-module"]
         "Export a module from the package."
         IT.exposedModules
         (\v flags -> flags { IT.exposedModules = v })
@@ -803,13 +928,13 @@ initCommand = CommandUI {
                                       ((Just . (:[])) `fmap` parse))
                           (fromMaybe [] . fmap (fmap display)))
 
-      , option [] ["sourceDir"]
+      , option [] ["source-dir"]
         "Directory containing package source."
         IT.sourceDirs (\v flags -> flags { IT.sourceDirs = v })
         (reqArg' "DIR" (Just . (:[]))
                        (fromMaybe []))
 
-      , option [] ["buildTool"]
+      , option [] ["build-tool"]
         "Required external build tool."
         IT.buildTools (\v flags -> flags { IT.buildTools = v })
         (reqArg' "TOOL" (Just . (:[]))
@@ -875,7 +1000,7 @@ readRepo = readPToMaybe parseRepo
 parseRepo :: Parse.ReadP r RemoteRepo
 parseRepo = do
   name <- Parse.munch1 (\c -> isAlphaNum c || c `elem` "_-.")
-  Parse.char ':'
+  _ <- Parse.char ':'
   uriStr <- Parse.munch1 (\c -> isAlphaNum c || c `elem` "+-=._/*()@'$:;&!?~")
   uri <- maybe Parse.pfail return (parseAbsoluteURI uriStr)
   return $ RemoteRepo {
