@@ -7,7 +7,7 @@
 --                    2008-2009 Duncan Coutts
 -- License     :  BSD3
 --
--- Maintainer  :  duncan@haskell.org
+-- Maintainer  :  duncan@community.haskell.org
 -- Portability :  portable
 --
 -- Reading, writing and manipulating \"@.tar@\" archive files.
@@ -51,23 +51,27 @@ module Distribution.Client.Tar (
 
   -- ** Sequences of tar entries
   Entries(..),
-  foldEntries,
-  unfoldEntries,
+  foldrEntries,
+  foldlEntries,
+  unfoldrEntries,
   mapEntries,
+  filterEntries,
+  entriesIndex,
 
   ) where
 
 import Data.Char     (ord)
 import Data.Int      (Int64)
-import Data.Bits     (Bits, shiftL)
+import Data.Bits     (Bits, shiftL, testBit)
 import Data.List     (foldl')
 import Numeric       (readOct, showOct)
-import Control.Monad (MonadPlus(mplus))
-
+import Control.Monad (MonadPlus(mplus), when)
+import qualified Data.Map as Map
 import qualified Data.ByteString.Lazy as BS
 import qualified Data.ByteString.Lazy.Char8 as BS.Char8
 import Data.ByteString.Lazy (ByteString)
 import qualified Codec.Compression.GZip as GZip
+import qualified Distribution.Client.GZipUtils as GZipUtils
 
 import System.FilePath
          ( (</>) )
@@ -79,6 +83,8 @@ import System.Directory
          , getPermissions, createDirectoryIfMissing, copyFile )
 import qualified System.Directory as Permissions
          ( Permissions(executable) )
+import Distribution.Compat.FilePerms
+         ( setFileExecutable )
 import System.Posix.Types
          ( FileMode )
 import System.Time
@@ -105,8 +111,8 @@ extractTarGzFile :: FilePath -- ^ Destination directory
                  -> FilePath -- ^ Expected subdir (to check for tarbombs)
                  -> FilePath -- ^ Tarball
                 -> IO ()
-extractTarGzFile dir expected tar =
-  unpack dir . checkTarbomb expected . read . GZip.decompress =<< BS.readFile tar
+extractTarGzFile dir expected tar = do
+  unpack dir . checkTarbomb expected . read . GZipUtils.maybeDecompress =<< BS.readFile tar
 
 --
 -- * Entry type
@@ -212,6 +218,9 @@ executableFilePermissions = 0o0755
 directoryPermissions :: Permissions
 directoryPermissions  = 0o0755
 
+isExecutable :: Permissions -> Bool
+isExecutable p = testBit p 0 || testBit p 6 -- user or other exectuable
+
 -- | An 'Entry' with all default values except for the file name and type. It
 -- uses the portable USTAR/POSIX format (see 'UstarHeader').
 --
@@ -381,24 +390,49 @@ data Entries = Next Entry Entries
              | Done
              | Fail String
 
-unfoldEntries :: (a -> Either String (Maybe (Entry, a))) -> a -> Entries
-unfoldEntries f = unfold
+unfoldrEntries :: (a -> Either String (Maybe (Entry, a))) -> a -> Entries
+unfoldrEntries f = unfold
   where
     unfold x = case f x of
       Left err             -> Fail err
       Right Nothing        -> Done
       Right (Just (e, x')) -> Next e (unfold x')
 
-foldEntries :: (Entry -> a -> a) -> a -> (String -> a) -> Entries -> a
-foldEntries next done fail' = fold
+foldrEntries :: (Entry -> a -> a) -> a -> (String -> a) -> Entries -> a
+foldrEntries next done fail' = fold
   where
     fold (Next e es) = next e (fold es)
     fold Done        = done
     fold (Fail err)  = fail' err
 
-mapEntries :: (Entry -> Either String Entry) -> Entries -> Entries
-mapEntries f =
-  foldEntries (\entry rest -> either Fail (flip Next rest) (f entry)) Done Fail
+foldlEntries :: (a -> Entry -> a) -> a -> Entries -> Either String a
+foldlEntries f = fold
+  where
+    fold a (Next e es) = (fold $! f a e) es
+    fold a Done        = Right a
+    fold _ (Fail err)  = Left err
+
+mapEntries :: (Entry -> Entry) -> Entries -> Entries
+mapEntries f = foldrEntries (Next . f) Done Fail
+
+filterEntries :: (Entry -> Bool) -> Entries -> Entries
+filterEntries p =
+  foldrEntries
+    (\entry rest -> if p entry
+                      then Next entry rest
+                      else rest)
+    Done Fail
+
+checkEntries :: (Entry -> Maybe String) -> Entries -> Entries
+checkEntries checkEntry =
+  foldrEntries
+    (\entry rest -> case checkEntry entry of
+                      Nothing  -> Next entry rest
+                      Just err -> Fail err)
+    Done Fail
+
+entriesIndex :: Entries -> Either String (Map.Map TarPath Entry)
+entriesIndex = foldlEntries (\m e -> Map.insert (entryTarPath e) e m) Map.empty
 
 --
 -- * Checking
@@ -462,16 +496,13 @@ checkEntryTarbomb expectedTopDir entry =
     _ -> Just $ "File in tar archive is not in the expected directory "
              ++ show expectedTopDir
 
-checkEntries :: (Entry -> Maybe String) -> Entries -> Entries
-checkEntries checkEntry =
-  mapEntries (\entry -> maybe (Right entry) Left (checkEntry entry))
 
 --
 -- * Reading
 --
 
 read :: ByteString -> Entries
-read = unfoldEntries getEntry
+read = unfoldrEntries getEntry
 
 getEntry :: ByteString -> Either String (Maybe (Entry, ByteString))
 getEntry bs
@@ -740,7 +771,7 @@ unpack baseDir entries = unpackEntries [] (checkSecurity entries)
     unpackEntries _     (Fail err)      = fail err
     unpackEntries links Done            = return links
     unpackEntries links (Next entry es) = case entryContent entry of
-      NormalFile file _ -> extractFile path file
+      NormalFile file _ -> extractFile entry path file
                         >> unpackEntries links es
       Directory         -> extractDir path
                         >> unpackEntries links es
@@ -750,12 +781,14 @@ unpack baseDir entries = unpackEntries [] (checkSecurity entries)
       where
         path = entryPath entry
 
-    extractFile path content = do
+    extractFile entry path content = do
       -- Note that tar archives do not make sure each directory is created
       -- before files they contain, indeed we may have to create several
       -- levels of directory.
       createDirectoryIfMissing True absDir
       BS.writeFile absPath content
+      when (isExecutable (entryPermissions entry))
+           (setFileExecutable absPath)
       where
         absDir  = baseDir </> FilePath.Native.takeDirectory path
         absPath = baseDir </> path
