@@ -11,7 +11,7 @@
 -- Entry point to the default cabal-install front-end.
 -----------------------------------------------------------------------------
 
-module Main where
+module Main (main) where
 
 import Distribution.Client.Setup
          ( GlobalFlags(..), globalCommand, globalRepos
@@ -19,15 +19,16 @@ import Distribution.Client.Setup
          , ConfigExFlags(..), configureExCommand
          , InstallFlags(..), defaultInstallFlags
          , installCommand, upgradeCommand
-         , fetchCommand, checkCommand
+         , FetchFlags(..), fetchCommand
+         , checkCommand
          , updateCommand
          , ListFlags(..), listCommand
          , InfoFlags(..), infoCommand
          , UploadFlags(..), uploadCommand
+         , ReportFlags(..), reportCommand
          , InitFlags, initCommand
          , reportCommand
-         , unpackCommand, UnpackFlags(..)
-         , parsePackageArgs )
+         , unpackCommand, UnpackFlags(..) )
 import Distribution.Simple.Setup
          ( BuildFlags(..), buildCommand
          , HaddockFlags(..), haddockCommand
@@ -39,12 +40,13 @@ import Distribution.Simple.Setup
          , TestFlags(..), testCommand
          , Flag(..), fromFlag, fromFlagOrDefault, flagToMaybe )
 
-import Distribution.Client.Types
-         ( UnresolvedDependency(UnresolvedDependency) )
 import Distribution.Client.SetupWrapper
          ( setupWrapper, SetupScriptOptions(..), defaultSetupScriptOptions )
 import Distribution.Client.Config
          ( SavedConfig(..), loadConfig, defaultConfigFile )
+import Distribution.Client.Targets
+         ( readUserTargets )
+
 import Distribution.Client.List             (list, info)
 import Distribution.Client.Install          (install, upgrade)
 import Distribution.Client.Configure        (configure)
@@ -59,8 +61,9 @@ import Distribution.Client.Init             (initCabal)
 import qualified Distribution.Client.Win32SelfUpgrade as Win32SelfUpgrade
 
 import Distribution.Simple.Compiler
-         ( PackageDB(..), PackageDBStack )
-import Distribution.Simple.Program (defaultProgramConfiguration)
+         ( Compiler, PackageDB(..), PackageDBStack )
+import Distribution.Simple.Program
+         ( ProgramConfiguration, defaultProgramConfiguration )
 import Distribution.Simple.Command
 import Distribution.Simple.Configure (configCompilerAux)
 import Distribution.Simple.Utils
@@ -68,7 +71,7 @@ import Distribution.Simple.Utils
 import Distribution.Text
          ( display )
 import Distribution.Verbosity as Verbosity
-       ( Verbosity, normal, intToVerbosity )
+       ( Verbosity, normal, intToVerbosity, lessVerbose )
 import qualified Paths_cabal_install (version)
 
 import System.Environment       (getArgs, getProgName)
@@ -112,9 +115,7 @@ mainWorker args = topHandler $
       putStr $ "\nYou can edit the cabal configuration file to set defaults:\n"
             ++ "  " ++ configFile ++ "\n"
     printOptionsList = putStr . unlines
-    printErrors errs = do
-      putStr (concat (intersperse "\n" errs))
-      exitFailure
+    printErrors errs = die $ concat (intersperse "\n" errs)
     printNumericVersion = putStrLn $ display Paths_cabal_install.version
     printVersion        = putStrLn $ "cabal-install version "
                                   ++ display Paths_cabal_install.version
@@ -123,19 +124,18 @@ mainWorker args = topHandler $
                                   ++ " of the Cabal library "
 
     commands =
-      [configureExCommand     `commandAddAction` configureAction
-      ,installCommand         `commandAddAction` installAction
+      [installCommand         `commandAddAction` installAction
+      ,updateCommand          `commandAddAction` updateAction
       ,listCommand            `commandAddAction` listAction
       ,infoCommand            `commandAddAction` infoAction
-      ,updateCommand          `commandAddAction` updateAction
-      ,upgradeCommand         `commandAddAction` upgradeAction
       ,fetchCommand           `commandAddAction` fetchAction
-      ,uploadCommand          `commandAddAction` uploadAction
+      ,unpackCommand          `commandAddAction` unpackAction
       ,checkCommand           `commandAddAction` checkAction
       ,sdistCommand           `commandAddAction` sdistAction
+      ,uploadCommand          `commandAddAction` uploadAction
       ,reportCommand          `commandAddAction` reportAction
-      ,unpackCommand          `commandAddAction` unpackAction
       ,initCommand            `commandAddAction` initAction
+      ,configureExCommand     `commandAddAction` configureAction
       ,wrapperAction (buildCommand defaultProgramConfiguration)
                      buildVerbosity    buildDistPref
       ,wrapperAction copyCommand
@@ -150,6 +150,7 @@ mainWorker args = topHandler $
                      regVerbosity      regDistPref
       ,wrapperAction testCommand
                      testVerbosity     testDistPref
+      ,upgradeCommand         `commandAddAction` upgradeAction
       ]
 
 wrapperAction :: Monoid flags
@@ -193,8 +194,8 @@ installAction (configFlags, _, installFlags) _ _globalFlags
 
 installAction (configFlags, configExFlags, installFlags)
               extraArgs globalFlags = do
-  pkgs <- either die return (parsePackageArgs extraArgs)
   let verbosity = fromFlagOrDefault normal (configVerbosity configFlags)
+  targets <- readUserTargets verbosity extraArgs
   config <- loadConfig verbosity (globalConfigFile globalFlags)
                                  (configUserInstall configFlags)
   let configFlags'   = savedConfigureFlags   config `mappend` configFlags
@@ -202,12 +203,11 @@ installAction (configFlags, configExFlags, installFlags)
       installFlags'  = defaultInstallFlags          `mappend`
                        savedInstallFlags     config `mappend` installFlags
       globalFlags'   = savedGlobalFlags      config `mappend` globalFlags
-  (comp, conf) <- configCompilerAux configFlags'
+  (comp, conf) <- configCompilerAux' configFlags'
   install verbosity
           (configPackageDB' configFlags') (globalRepos globalFlags')
-          comp conf configFlags' configExFlags' installFlags'
-          [ UnresolvedDependency pkg (configConfigurationsFlags configFlags')
-          | pkg <- pkgs ]
+          comp conf globalFlags' configFlags' configExFlags' installFlags'
+          targets
 
 listAction :: ListFlags -> [String] -> GlobalFlags -> IO ()
 listAction listFlags extraArgs globalFlags = do
@@ -215,7 +215,7 @@ listAction listFlags extraArgs globalFlags = do
   config <- loadConfig verbosity (globalConfigFile globalFlags) mempty
   let configFlags  = savedConfigureFlags config
       globalFlags' = savedGlobalFlags    config `mappend` globalFlags
-  (comp, conf) <- configCompilerAux configFlags
+  (comp, conf) <- configCompilerAux' configFlags
   list verbosity
        (configPackageDB' configFlags)
        (globalRepos globalFlags')
@@ -226,8 +226,8 @@ listAction listFlags extraArgs globalFlags = do
 
 infoAction :: InfoFlags -> [String] -> GlobalFlags -> IO ()
 infoAction infoFlags extraArgs globalFlags = do
-  pkgs <- either die return (parsePackageArgs extraArgs)
   let verbosity = fromFlag (infoVerbosity infoFlags)
+  targets <- readUserTargets verbosity extraArgs
   config <- loadConfig verbosity (globalConfigFile globalFlags) mempty
   let configFlags  = savedConfigureFlags config
       globalFlags' = savedGlobalFlags    config `mappend` globalFlags
@@ -237,8 +237,9 @@ infoAction infoFlags extraArgs globalFlags = do
        (globalRepos globalFlags')
        comp
        conf
+       globalFlags'
        infoFlags
-       [ UnresolvedDependency pkg []  | pkg <- pkgs ]
+       targets
 
 updateAction :: Flag Verbosity -> [String] -> GlobalFlags -> IO ()
 updateAction verbosityFlag extraArgs globalFlags = do
@@ -253,34 +254,33 @@ upgradeAction :: (ConfigFlags, ConfigExFlags, InstallFlags)
               -> [String] -> GlobalFlags -> IO ()
 upgradeAction (configFlags, configExFlags, installFlags)
               extraArgs globalFlags = do
-  pkgs <- either die return (parsePackageArgs extraArgs)
   let verbosity = fromFlagOrDefault normal (configVerbosity configFlags)
+  targets <- readUserTargets verbosity extraArgs
   config <- loadConfig verbosity (globalConfigFile globalFlags)
                                  (configUserInstall configFlags)
   let configFlags'   = savedConfigureFlags   config `mappend` configFlags
       configExFlags' = savedConfigureExFlags config `mappend` configExFlags
-      installFlags'  = savedInstallFlags     config `mappend` installFlags
+      installFlags'  = defaultInstallFlags          `mappend`
+                       savedInstallFlags     config `mappend` installFlags
       globalFlags'   = savedGlobalFlags      config `mappend` globalFlags
-  (comp, conf) <- configCompilerAux configFlags'
+  (comp, conf) <- configCompilerAux' configFlags'
   upgrade verbosity
           (configPackageDB' configFlags') (globalRepos globalFlags')
-          comp conf configFlags' configExFlags' installFlags'
-          [ UnresolvedDependency pkg (configConfigurationsFlags configFlags')
-          | pkg <- pkgs ]
+          comp conf globalFlags' configFlags' configExFlags' installFlags'
+          targets
 
-fetchAction :: Flag Verbosity -> [String] -> GlobalFlags -> IO ()
-fetchAction verbosityFlag extraArgs globalFlags = do
-  pkgs <- either die return (parsePackageArgs extraArgs)
-  let verbosity = fromFlag verbosityFlag
+fetchAction :: FetchFlags -> [String] -> GlobalFlags -> IO ()
+fetchAction fetchFlags extraArgs globalFlags = do
+  let verbosity = fromFlag (fetchVerbosity fetchFlags)
+  targets <- readUserTargets verbosity extraArgs
   config <- loadConfig verbosity (globalConfigFile globalFlags) mempty
   let configFlags  = savedConfigureFlags config
       globalFlags' = savedGlobalFlags config `mappend` globalFlags
-  (comp, conf) <- configCompilerAux configFlags
+  (comp, conf) <- configCompilerAux' configFlags
   fetch verbosity
         (configPackageDB' configFlags) (globalRepos globalFlags')
-        comp conf
-        [ UnresolvedDependency pkg [] --TODO: flags?
-        | pkg <- pkgs ]
+        comp conf globalFlags' fetchFlags
+        targets
 
 uploadAction :: UploadFlags -> [String] -> GlobalFlags -> IO ()
 uploadAction uploadFlags extraArgs globalFlags = do
@@ -328,28 +328,38 @@ sdistAction sflags extraArgs _globalFlags = do
     die $ "'sdist' doesn't take any extra arguments: " ++ unwords extraArgs
   sdist sflags
 
-reportAction :: Flag Verbosity -> [String] -> GlobalFlags -> IO ()
-reportAction verbosityFlag extraArgs globalFlags = do
+reportAction :: ReportFlags -> [String] -> GlobalFlags -> IO ()
+reportAction reportFlags extraArgs globalFlags = do
   unless (null extraArgs) $ do
     die $ "'report' doesn't take any extra arguments: " ++ unwords extraArgs
 
-  let verbosity = fromFlag verbosityFlag
+  let verbosity = fromFlag (reportVerbosity reportFlags)
   config <- loadConfig verbosity (globalConfigFile globalFlags) mempty
   let globalFlags' = savedGlobalFlags config `mappend` globalFlags
+      reportFlags' = savedReportFlags config `mappend` reportFlags
 
   Upload.report verbosity (globalRepos globalFlags')
+    (flagToMaybe $ reportUsername reportFlags')
+    (flagToMaybe $ reportPassword reportFlags')
 
 unpackAction :: UnpackFlags -> [String] -> GlobalFlags -> IO ()
-unpackAction flags extraArgs globalFlags = do
-  pkgs <- either die return (parsePackageArgs extraArgs)
-  let verbosity = fromFlag (unpackVerbosity flags)
+unpackAction unpackFlags extraArgs globalFlags = do
+  let verbosity = fromFlag (unpackVerbosity unpackFlags)
+  targets <- readUserTargets verbosity extraArgs
   config <- loadConfig verbosity (globalConfigFile globalFlags) mempty
-  unpack flags (globalRepos (savedGlobalFlags config)) pkgs
+  let globalFlags' = savedGlobalFlags config `mappend` globalFlags
+  unpack verbosity
+         (globalRepos (savedGlobalFlags config))
+         globalFlags'
+         unpackFlags
+         targets
 
 initAction :: InitFlags -> [String] -> GlobalFlags -> IO ()
 initAction flags _extraArgs _globalFlags = do
   initCabal flags
 
+-- | See 'Distribution.Client.Install.withWin32SelfUpgrade' for details.
+--
 win32SelfUpgradeAction :: [String] -> IO ()
 win32SelfUpgradeAction (pid:path:rest) =
   Win32SelfUpgrade.deleteOldExeFile verbosity (read pid) path
@@ -384,3 +394,10 @@ configPackageDB' cfg =
   implicitPackageDbStack userInstall (flagToMaybe (configPackageDB cfg))
   where
     userInstall = fromFlagOrDefault True (configUserInstall cfg)
+
+configCompilerAux' :: ConfigFlags
+                   -> IO (Compiler, ProgramConfiguration)
+configCompilerAux' configFlags =
+  configCompilerAux configFlags
+    --FIXME: make configCompilerAux use a sensible verbosity
+    { configVerbosity = fmap lessVerbose (configVerbosity configFlags) }
